import java.awt.Point;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.SocketException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;

public class MainClass {
	static ArrayList<Connection> conList = new ArrayList<Connection>();
	static ServerSocket srvSo;
	static ArrayList<Integer> shipList = new ArrayList<Integer>();
	static int serverPort = 0;
	static int iterationCount = 0;
	static String runningWord = "SERVER IS RUNNING  ";

	public static void main(String[] args) throws IOException, ClassNotFoundException {

		String ships = null;
		int fieldsize = 10;
		try {
			for (int i = 0; i < args.length; i++) {
				if (args[i].contains("--port")) {
					i++;
					serverPort = Integer.parseInt(args[i]);
				}
				if (args[i].contains("--ships")) {
					i++;
					ships = args[i];
				}
				if (args[i].contains("--fieldsize")) {
					i++;
					fieldsize = Integer.parseInt(args[i]);
				}
			}
		} catch (IndexOutOfBoundsException e) {
			errorOnStart();
		}
		if (serverPort < 1000 || serverPort > 64000) errorOnStart();
		for (int i = 0; i < ships.length(); i++) {
			int number = Integer.parseInt(String.valueOf(ships.charAt(i)));
			if (number > 1 || number <= 20) {
				shipList.add(number);
			}
		}
		if (fieldsize < 4 || fieldsize > 20) errorOnStart();
		
		srvSo = new ServerSocket(serverPort);

		printScreen("Server gestartet");
		
		int z = 0;
		while (true) {
			try {
				while (conList.size() < 2) {
					printScreen("");
					printScreen("Warte auf eingehende Verbindung...");
					conList.add(new Connection(srvSo.accept()));
					printScreen(conList.size() + ". Verbindung angenommen");
					if (conList.size() == 2) {
						boolean beginner = true;
						for (Connection item : conList) {
							item.setOutSocket(getOppositeConnection(item).inSocket);
							item.sendGameInfo(new GameInfo(new Point(fieldsize, fieldsize), shipList, beginner, item.name));
							item.start();
							beginner = false;
						}
					}
				}
				if (z == Integer.MAX_VALUE) {
					z = 0;
					System.out.print(runningWord.charAt(iterationCount % runningWord.length()));
					System.out.print((char) 8);
					iterationCount++;
					if (runningWord.length() == iterationCount) iterationCount = 0;
				}
				z++;
			} catch (SocketException e) {printScreen("Alle Verbindungen verloren...");}
		}
	}

	public static Connection getOppositeConnection(Connection con) {
		if (con == conList.get(0))
			return conList.get(1);
		return conList.get(0);
	}
	
	public static void printScreen(String data) {
		LocalDateTime t = LocalDateTime.now();
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss.SS");
		String formattedDate = t.format(formatter);
		String message = "";
		message = formattedDate + " | " + fillStringTo(" ", 28) + " |   | Server | " + data;
		System.out.println(message);
	}

	public static void printError(String data) {
		LocalDateTime t = LocalDateTime.now();
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm:ss.SS");
		String formattedDate = t.format(formatter);
		String message = "";
		message = formattedDate + " | " + fillStringTo(" ", 28) + " |   | Server | " + data;
		System.err.println(message);
	}
	
	private static String fillStringTo(String text, int count) {
		for (int i = text.length(); i <= count; i++) {
			text += " ";
		}
		return text.substring(0, count);
	}

	private static void errorOnStart() {
		printError("Bitte alle Argumente angeben!");
		printError("--port [Integer]      Min: 1000, Max: 64000");
		printError("--ships [IntegerArray] MinArrayLength: 1, MaxArrayLength: 9, MinShipLength: 2, MinShipLength: 9");
		printError("--fieldsize [Integer]  Min: 4, Max: 20");
		System.exit(0);
	}
	
}
